#!/usr/bin/env python3
"""translatedoc - ドキュメントを翻訳するツール。"""
import argparse
import os
import pathlib
import sys
import typing

import openai
import tqdm

if typing.TYPE_CHECKING:
    from unstructured.documents.elements import Element


def main():
    """メイン関数。"""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--output-dir",
        "-o",
        default=pathlib.Path("."),
        type=pathlib.Path,
        help="output directory (default: .)",
    )
    parser.add_argument(
        "--force", "-f", action="store_true", help="overwrite existing files"
    )
    parser.add_argument(
        "--language",
        "-l",
        default="Japanese",
        help="target language name (default: Japanese)",
    )
    parser.add_argument(
        "--model",
        "-m",
        default="gpt-3.5-turbo-1106",
        help="model (default: gpt-3.5-turbo-1106)",
    )
    parser.add_argument(
        "--strategy",
        "-s",
        choices=["auto", "fast", "ocr_only", "hi_res"],
        default="hi_res",
        help="document partitioning strategy (default: hi_res)",
        # hi_resはtesseractやdetectron2を使うので重いけど精度が高いのでデフォルトに
    )
    parser.add_argument(
        "--chunk-max-characters",
        type=int,
        default=4096,
        help="document chunk size (default: 4096)",
    )
    parser.add_argument("input_files", nargs="+", help="input files/URLs")
    args = parser.parse_args()

    openai_client = openai.OpenAI(base_url=os.environ.get("OPENAI_API_URL"))

    exit_code = 0
    for input_file in tqdm.tqdm(args.input_files, desc="Input files/URLs"):
        input_path = pathlib.Path(input_file)
        try:
            # ドキュメントの読み込み・パース
            chunks = _load_document(input_file, args)
            source_name = input_path.with_suffix(".Source.txt").name
            source_path = args.output_dir / source_name
            if not _check_overwrite(source_path, args.force):
                continue
            source_path.parent.mkdir(parents=True, exist_ok=True)
            source_path.write_text("\n\n".join(str(c).strip() for c in chunks) + "\n\n")

            # 動作確認用: --language=noneで翻訳をスキップ
            if args.language.lower() == "none":
                continue

            # 翻訳
            output_name = input_path.with_suffix(f".{args.language}.txt").name
            output_path = args.output_dir / output_name
            if not _check_overwrite(output_path, args.force):
                continue
            output_path.parent.mkdir(parents=True, exist_ok=True)
            with output_path.open("w") as file:
                for chunk in tqdm.tqdm(chunks, desc="Chunks"):
                    output_chunk = _translate(str(chunk), args, openai_client)
                    file.write(output_chunk.strip() + "\n\n")
                    file.flush()
            tqdm.tqdm.write(f"{output_path} written.")
        except Exception as e:
            print(f"Error: {e} ({input_file})", file=sys.stderr)
            exit_code = 1

    sys.exit(exit_code)


def _check_overwrite(output_path: pathlib.Path, force: bool) -> bool:
    """上書き確認。"""
    if output_path.exists() and not force:
        with tqdm.tqdm.external_write_mode():
            print(f"Output path already exists: {output_path}", file=sys.stderr)
            try:
                input_ = input("Overwrite? [y/N] ")
            except EOFError:
                input_ = ""
            if input_ != "y":
                print("Skipped.", file=sys.stderr)
                return False
    return True


def _load_document(input_file: str, args: argparse.Namespace) -> "list[Element]":
    """ドキュメントの読み込み・パース。"""
    with tqdm.tqdm.external_write_mode():
        from unstructured.chunking.title import chunk_by_title
        from unstructured.partition.auto import partition

    kwargs = (
        {"url": input_file}
        if input_file.startswith("http://") or input_file.startswith("https://")
        else {"filename": input_file}
    )
    elements = partition(**kwargs, strategy=args.strategy)
    chunks = chunk_by_title(
        elements,
        combine_text_under_n_chars=args.chunk_max_characters // 4,
        new_after_n_chars=args.chunk_max_characters // 2,
        max_characters=args.chunk_max_characters,
    )
    return chunks


def _translate(
    chunk: str, args: argparse.Namespace, openai_client: openai.OpenAI
) -> str:
    """翻訳。"""
    response = openai_client.chat.completions.create(
        model=args.model,
        messages=[
            {
                "role": "system",
                "content": f"Translate the input into {args.language}."
                " Do not output anything other than the translation result."
                " Do not translate names of people, mathematical formulas,"
                " source code, URLs, etc.",
            },
            {"role": "user", "content": chunk},
        ],
        temperature=0.0,
    )
    if len(response.choices) != 1 or response.choices[0].message.content is None:
        return f"*** Unexpected response: {response.dict()=} ***"
    return response.choices[0].message.content


if __name__ == "__main__":
    main()
